@@ -396,7 +396,7 @@ class RulesCollection:
         else:
             self.options = options
         self.profile = []
-        self.app = app or get_app(offline=True)
+        self.app = app or get_app(cached=True)
 
         if profile_name:
             self.profile = PROFILES[profile_name]
@@ -457,7 +457,7 @@ class RulesCollection:
         """Return a rule from inside the collection based on its id."""
         if not isinstance(item, str):
             msg = f"Expected str but got {type(item)} when trying to access rule by it's id"
-            raise RuntimeError(msg)
+            raise TypeError(msg)
         for rule in self.rules:
             if rule.id == item:
                 return rule
@@ -503,11 +503,16 @@ class RulesCollection:
                 or rule.has_dynamic_tags
                 or not set(rule.tags).union([rule.id]).isdisjoint(tags)
             ):
-                _logger.debug("Running rule %s", rule.id)
-                rule_definition = set(rule.tags)
-                rule_definition.add(rule.id)
-                if set(rule_definition).isdisjoint(skip_list):
-                    matches.extend(rule.getmatches(file))
+                if tags and set(rule.tags).union(list(rule.ids().keys())).isdisjoint(
+                    tags,
+                ):
+                    _logger.debug("Skipping rule %s", rule.id)
+                else:
+                    _logger.debug("Running rule %s", rule.id)
+                    rule_definition = set(rule.tags)
+                    rule_definition.add(rule.id)
+                    if set(rule_definition).isdisjoint(skip_list):
+                        matches.extend(rule.getmatches(file))
             else:
                 _logger.debug("Skipping rule %s", rule.id)
 
