@@ -106,6 +106,8 @@ class Runner:
             checked_files = set()
         self.checked_files = checked_files
 
+        self.app = get_app(cached=True)
+
     def _update_exclude_paths(self, exclude_paths: list[str]) -> None:
         if exclude_paths:
             # These will be (potentially) relative paths
@@ -172,7 +174,7 @@ class Runner:
                         match = MatchError(
                             message=warn.source.message or warn.category.__name__,
                             rule=self.rules["warning"],
-                            filename=warn.source.filename.filename,
+                            lintable=Lintable(warn.source.filename.filename),
                             tag=warn.source.tag,
                             lineno=warn.source.lineno,
                         )
@@ -183,7 +185,7 @@ class Runner:
                                 warn.message if isinstance(warn.message, str) else "?"
                             ),
                             rule=self.rules["warning"],
-                            filename=str(filename),
+                            lintable=Lintable(str(filename)),
                         )
                     matches.append(match)
                     continue
@@ -232,12 +234,12 @@ class Runner:
 
         # -- phase 1 : syntax check in parallel --
         if not self.skip_ansible_syntax_check:
-            app = get_app(offline=True)
+            # app = get_app(cached=True)
 
             def worker(lintable: Lintable) -> list[MatchError]:
                 return self._get_ansible_syntax_check_matches(
                     lintable=lintable,
-                    app=app,
+                    app=self.app,
                 )
 
             for lintable in self.lintables:
@@ -260,28 +262,28 @@ class Runner:
                 matches.extend(data)
 
             matches = self._filter_excluded_matches(matches)
+
         # -- phase 2 ---
-        if not matches:
-            # do our processing only when ansible syntax check passed in order
-            # to avoid causing runtime exceptions. Our processing is not as
-            # resilient to be able process garbage.
-            matches.extend(self._emit_matches(files))
+        # do our processing only when ansible syntax check passed in order
+        # to avoid causing runtime exceptions. Our processing is not as
+        # resilient to be able process garbage.
+        matches.extend(self._emit_matches(files))
 
-            # remove duplicates from files list
-            files = [value for n, value in enumerate(files) if value not in files[:n]]
+        # remove duplicates from files list
+        files = [value for n, value in enumerate(files) if value not in files[:n]]
 
-            for file in self.lintables:
-                if file in self.checked_files or not file.kind:
-                    continue
-                _logger.debug(
-                    "Examining %s of type %s",
-                    ansiblelint.file_utils.normpath(file.path),
-                    file.kind,
-                )
+        for file in self.lintables:
+            if file in self.checked_files or not file.kind or file.failed():
+                continue
+            _logger.debug(
+                "Examining %s of type %s",
+                ansiblelint.file_utils.normpath(file.path),
+                file.kind,
+            )
 
-                matches.extend(
-                    self.rules.run(file, tags=set(self.tags), skip_list=self.skip_list),
-                )
+            matches.extend(
+                self.rules.run(file, tags=set(self.tags), skip_list=self.skip_list),
+            )
 
         # update list of checked files
         self.checked_files.update(self.lintables)
@@ -352,11 +354,11 @@ class Runner:
             # https://github.com/ansible/ansible-lint/issues/3650
             env["ANSIBLE_INVENTORY_ANY_UNPARSED_IS_FAILED"] = "False"
 
-            run = subprocess.run(
+            run = subprocess.run(  # noqa: S603
                 cmd,
                 stdin=subprocess.PIPE,
                 capture_output=True,
-                shell=False,  # needed when command is a list # noqa: S603
+                shell=False,  # needed when command is a list
                 text=True,
                 check=False,
                 env=env,
@@ -387,13 +389,23 @@ class Runner:
                     details = groups.get("details", "")
                     lineno = int(groups.get("line", 1))
 
-                    if "filename" in groups:
+                    if (
+                        "filename" in groups
+                        and str(lintable.path.absolute()) != groups["filename"]
+                        and lintable.filename != groups["filename"]
+                    ):
+                        # avoids creating a new lintable object if the filename
+                        # is matching as this might prevent Lintable.failed()
+                        # feature from working well.
                         filename = Lintable(groups["filename"])
                     else:
                         filename = lintable
                     column = int(groups.get("column", 1))
 
-                    if pattern.tag == "unknown-module" and app.options.nodeps:
+                    if (
+                        pattern.tag in ("unknown-module", "specific")
+                        and app.options.nodeps
+                    ):
                         ignore_rc = True
                     else:
                         results.append(
@@ -443,6 +455,9 @@ class Runner:
         visited: set[Lintable] = set()
         while visited != self.lintables:
             for lintable in self.lintables - visited:
+                visited.add(lintable)
+                if not lintable.path.exists():
+                    continue
                 try:
                     children = self.find_children(lintable)
                     for child in children:
@@ -456,8 +471,10 @@ class Runner:
                     exc.rule = self.rules["load-failure"]
                     yield exc
                 except AttributeError:
-                    yield MatchError(lintable=lintable, rule=self.rules["load-failure"])
-                visited.add(lintable)
+                    yield MatchError(
+                        lintable=lintable,
+                        rule=self.rules["load-failure"],
+                    )
 
     def find_children(self, lintable: Lintable) -> list[Lintable]:
         """Traverse children of a single file or folder."""
@@ -476,7 +493,9 @@ class Runner:
             try:
                 playbook_ds = ansiblelint.utils.parse_yaml_from_file(str(lintable.path))
             except AnsibleError as exc:
-                raise SystemExit(exc) from exc
+                msg = f"Loading {lintable.filename} caused an {type(exc).__name__} exception: {exc}, file was ignored."
+                logging.exception(msg)
+                return []
         results = []
         # playbook_ds can be an AnsibleUnicode string, which we consider invalid
         if isinstance(playbook_ds, str):
@@ -484,7 +503,7 @@ class Runner:
         for item in ansiblelint.utils.playbook_items(playbook_ds):
             # if lintable.kind not in ["playbook"]:
             for child in self.play_children(
-                lintable.path.parent,
+                lintable,
                 item,
                 lintable.kind,
                 playbook_dir,
@@ -505,25 +524,24 @@ class Runner:
                 if path != path_str:
                     child.path = Path(path)
                     child.name = child.path.name
-
                 results.append(child)
         return results
 
     def play_children(
         self,
-        basedir: Path,
+        lintable: Lintable,
         item: tuple[str, Any],
         parent_type: FileType,
         playbook_dir: str,
     ) -> list[Lintable]:
         """Flatten the traversed play tasks."""
         # pylint: disable=unused-argument
-
-        handlers = HandleChildren(self.rules)
+        basedir = lintable.path.parent
+        handlers = HandleChildren(self.rules, app=self.app)
 
         delegate_map: dict[
             str,
-            Callable[[str, Any, Any, FileType], list[Lintable]],
+            Callable[[Lintable, Any, Any, FileType], list[Lintable]],
         ] = {
             "tasks": handlers.taskshandlers_children,
             "pre_tasks": handlers.taskshandlers_children,
@@ -551,7 +569,7 @@ class Runner:
                 {"playbook_dir": PLAYBOOK_DIR or str(basedir.resolve())},
                 fail_on_undefined=False,
             )
-            return delegate_map[k](str(basedir), k, v, parent_type)
+            return delegate_map[k](lintable, k, v, parent_type)
         return []
 
     def plugin_children(self, lintable: Lintable) -> list[Lintable]:
